@@ -1,6 +1,6 @@
 #
 # Catlyst::Action::SerializeBase.pm
-# Created by: Adam Jacob, Marchex, <adam@hjksolutions.com>
+# Created by: Adam Jacob, Marchex, <adam@marchex.com>
 #
 # $Id$
 
@@ -12,17 +12,7 @@ use warnings;
 use base 'Catalyst::Action';
 use Module::Pluggable::Object;
 use Data::Dump qw(dump);
-use Catalyst::Request::REST;
-use Catalyst::Utils ();
-
-sub new {
-  my $class  = shift;
-  my $config = shift;
-  Catalyst::Request::REST->_insert_self_into(
-    Catalyst::Utils::class2appclass($config->{class})
-  );
-  return $class->SUPER::new($config, @_);
-}
+use HTTP::Headers::Util qw(split_header_words);
 
 __PACKAGE__->mk_accessors(qw(_serialize_plugins _loaded_plugins));
 
@@ -43,51 +33,55 @@ sub _load_content_plugins {
         $self->_serialize_plugins( \@plugins );
     }
 
+    # First, we use the content type in the HTTP Request.  It wins all.
+    my $content_type = $c->request->content_type || '';
+
+    # Second, we allow GET requests to tunnel the content-type as
+    #  a query param.
+    if (!$content_type && $c->req->method eq "GET" && $c->req->param('content-type')) {
+
+        $content_type = $c->req->param('content-type');
+    }
+
+    # Third, we parse the Accept header, and see if the client
+    # takes a format we understand.
+    #
+    # This is taken from chansen's Apache2::UploadProgress.
+    my $used_accept = 0;
+    if ( !$content_type && $c->request->header('Accept') ) {
+        my $accept_header = $c->request->header('Accept');
+        my %accept        = ();
+        my $counter       = 0;
+
+        foreach my $pair ( split_header_words($accept_header) ) {
+
+            my ( $type, $qvalue ) = @{$pair}[ 0, 3 ];
+
+            unless ( defined $qvalue ) {
+                $qvalue = 1 - ( ++$counter / 1000 );
+            }
+
+            $accept{$type} = sprintf( '%.3f', $qvalue );
+        }
+
+        foreach my $type ( sort { $accept{$b} <=> $accept{$a} } keys %accept )
+        {
+
+            if ( exists $controller->config->{'serialize'}->{'map'}->{$type} )
+            {
+                $content_type = $type;
+                last;
+            }
+        }
+        $used_accept = 1;
+    }
+
     # Finally, we load the class.  If you have a default serializer,
     # and we still don't have a content-type that exists in the map,
     # we'll use it.
     my $sclass = $search_path . "::";
     my $sarg;
-    my $map;
-
-    my $config;
-    
-    if ( exists $controller->{'serialize'} ) {
-        $c->log->info("Using deprecated configuration for Catalyst::Action::REST!");
-        $c->log->info("Please see perldoc Catalyst::Action::REST for the update guide");
-        $config = $controller->{'serialize'};
-    } else {
-        $config = $controller;
-    }
-    $map = $config->{'map'};
-
-    # pick preferred content type
-    my @accepted_types; # priority order, best first
-    # give top priority to content type specified by stash, if any
-    my $content_type_stash_key = $config->{content_type_stash_key};
-    if ($content_type_stash_key
-        and my $stashed = $c->stash->{$content_type_stash_key}
-    ) {
-        # convert to array if not already a ref
-        $stashed = [ $stashed ] if not ref $stashed;
-        push @accepted_types, @$stashed;
-    }
-    # then content types requested by caller
-    push @accepted_types, @{ $c->request->accepted_content_types };
-    # then the default
-    push @accepted_types, $config->{'default'} if $config->{'default'};
-    # pick the best match that we have a serializer mapping for
-    my ($content_type) = grep { $map->{$_} } @accepted_types;
-
-    return $self->_unsupported_media_type($c, $content_type)
-        if not $content_type;
-
-    # carp about old text/x-json
-    if ($content_type eq 'text/x-json') {
-        $c->log->info('Using deprecated text/x-json content-type.');
-        $c->log->info('Use application/json instead!');
-    }
-
+    my $map = $controller->config->{'serialize'}->{'map'};
     if ( exists( $map->{$content_type} ) ) {
         my $mc;
         if ( ref( $map->{$content_type} ) eq "ARRAY" ) {
@@ -108,7 +102,11 @@ sub _load_content_plugins {
             return $self->_unsupported_media_type($c, $content_type);
         }
     } else {
-        return $self->_unsupported_media_type($c, $content_type);
+        if ( exists( $controller->config->{'serialize'}->{'default'} ) ) {
+            $sclass .= $controller->config->{'serialize'}->{'default'};
+        } else {
+            return $self->_unsupported_media_type($c, $content_type);
+        }
     }
     unless ( exists( $self->_loaded_plugins->{$sclass} ) ) {
         my $load_class = $sclass;
@@ -117,7 +115,8 @@ sub _load_content_plugins {
         eval { require $load_class; };
         if ($@) {
             $c->log->error(
-                "Error loading $sclass for " . $content_type . ": $!" );
+                "Error loading $sclass for " . $content_type . ": $!" )
+              if $c->log->is_debug;
             return $self->_unsupported_media_type($c, $content_type);
         } else {
             $self->_loaded_plugins->{$sclass} = 1;
@@ -127,7 +126,7 @@ sub _load_content_plugins {
     if ($search_path eq "Catalyst::Action::Serialize") {
         if ($content_type) {
             $c->response->header( 'Vary' => 'Content-Type' );
-        } elsif ($c->request->accept_only) {
+        } elsif ($used_accept) {
             $c->response->header( 'Vary' => 'Accept' );
         }
         $c->response->content_type($content_type);
@@ -140,7 +139,7 @@ sub _unsupported_media_type {
     my ( $self, $c, $content_type ) = @_;
     $c->res->content_type('text/plain');
     $c->res->status(415);
-    if (defined($content_type) && $content_type ne "") {
+    if (defined($content_type)) {
         $c->res->body(
             "Content-Type " . $content_type . " is not supported.\r\n" );
     } else {
